﻿using PGT.Common;
using PGT.ConfigurationManager.ConfigDSTableAdapters;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using System.Security.Cryptography;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Xml;

namespace PGT.ConfigurationManager
{
  public partial class ConfigManager : Form
  {
    #region Fields
    private BackgroundWorker _workInProgress;
    private string _workInProgressText = "Work in progress...";
    private string _workInProgressCaption = "Please be patient";
    private SqlCommand _workInProgressSqlCommandToCancel = null;
    /// <summary>
    /// This indicates whether to display a cancel button in the WorkAnimation window
    /// </summary>
    private bool _workInProgressSupportCancellation = false;
    /// <summary>
    /// This is set to true when user clicks the cancel button in the WorkAnimation dialoge
    /// </summary>
    private bool _workInProgress_CancelPressed = false;
    /// <summary>
    /// Indicates whether the configuration text in tbConfiguration has changed since loaded frmom database
    /// </summary>
    private bool targetConfigurationChanged;
    /// <summary>
    /// The ConfigurationSetTargetID of the displayed configuration in tbConfigurationLines. Set when loaded.
    /// </summary>
    private int loadedConfigSetTargetID = -1;
    #endregion

    #region Initialization
    public ConfigManager()
    {
      InitializeComponent();
      _workInProgress = new BackgroundWorker();
      _workInProgress.DoWork += DoWorkAnimation;
      _workInProgress.WorkerSupportsCancellation = true;
      SetControlBackGround(this);
    }
    private void ConfigManager_Load(object sender, EventArgs e)
    {
      _workInProgressCaption = "Connecting to database";
      _workInProgressText = "Now connecting, please wait...";
      if (!_workInProgress.IsBusy) _workInProgress.RunWorkerAsync();
      try
      {
        try
        {
          tsslSQLServer.Text = Properties.Settings.Default.SQLServerName + " : " + Properties.Settings.Default.DatabaseName;
          UpdateConfigTargetList();
          UpdateConfigSetList();
        }
        catch (System.Data.SqlClient.SqlException Ex)
        {
          _workInProgress.CancelAsync();
          if (MessageBox.Show(string.Format("An unexpected database error occurred : {0} \r\n Do you want to reconfigure SQL connection parameters ? ", Ex.Message), "Database connection error", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == System.Windows.Forms.DialogResult.Yes)
          {
            (new SqlConnectionEditor(_workInProgress)).ShowDialog();
          }
        }
        catch (Exception Ex)
        {
          _workInProgress.CancelAsync();
          MessageBox.Show(string.Format("An unexpected database error occurred : {0}", Ex.Message), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
        this.Text += string.Format(" - V{0}", Assembly.GetExecutingAssembly().GetName().Version.ToString());
      }
      finally
      {
        _workInProgress.CancelAsync();
      }
    }
    private void SetControlBackGround(Control rootControl)
    {
      if (rootControl != null)
      {
        foreach (Control thisControl in rootControl.Controls)
        {
          if (thisControl is System.Windows.Forms.Label || thisControl is CheckBox || thisControl is GroupBox)
          {
            thisControl.BackColor = System.Drawing.Color.Transparent;
            MethodInfo m = thisControl.GetType().GetMethod("SetStyle");
            if (m != null) m.Invoke(thisControl, new object[] { ControlStyles.SupportsTransparentBackColor, true });
          }
          SetControlBackGround(thisControl);
        }
      }

    }

    #endregion

    #region General stuff
    private void DoWorkAnimation(object sender, DoWorkEventArgs e)
    {
      WorkInProgressAnimation L = null;
      _workInProgress_CancelPressed = false;
      DateTime waitStartedAt = DateTime.Now;
      while (true)
      {
        System.Threading.Thread.Sleep(20);
        if (L == null || (DateTime.Now - waitStartedAt).TotalMilliseconds > 500)
        {
          if (L == null)
          {
            L = new WorkInProgressAnimation(_workInProgressText, _workInProgressCaption, _workInProgressSupportCancellation);
            L.SqlCommandToCancel = _workInProgressSqlCommandToCancel;
            L.TopMost = true;
            L.Show();
            L.BringToFront();
            L.Focus();
          }
          else
          {
            L.Text = _workInProgressCaption;
            L.SetLabelText(_workInProgressText);
            L.BringToFront();
            _workInProgress_CancelPressed = L.CancelPressed;
          }

        }
        Application.DoEvents();
        if (_workInProgress.CancellationPending)
        {
          if (L != null) L.Close();
          break;
        }
      }
    }

    #endregion

    #region Manage Config Targets
    /// <summary>
    /// Refreshes the ConfigTargets listview from database
    /// </summary>
    private void UpdateConfigTargetList()
    {
      configTargetsTA.Fill(this.configDS.ConfigTargets);
      lvTargets.BeginUpdate();
      try
      {
        lvTargets.Items.Clear();
        lvTargets.ListViewItemSorter = null;
        foreach (var item in this.configDS.ConfigTargets)
        {
          ListViewItem lvi = new ListViewItem();
          lvi.Text = item.TargetName;
          lvi.SubItems.Add(item.TargetIP);
          lvi.SubItems.Add(item.DeviceVendor);
          lvi.SubItems.Add(item.IsJumpServerIPNull() ? "" : item.JumpServerIP);
          lvi.SubItems.Add(item.Protocol);
          lvi.SubItems.Add(item.IsPortNull() ? "" : item.Port.ToString());
          lvi.SubItems.Add(item.IsNoteNull() ? "" : item.Note);
          lvi.Tag = item.ConfigTargetID;
          lvTargets.Items.Add(lvi);
        }
        lvTargets.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
      }
      finally
      {
        lvTargets.EndUpdate();
      }
    }
    /// <summary>
    /// Adds a new configuration target
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void AddConfigTarget()
    {
      ConfigTarget CT = new ConfigTarget();
      if (CT.ShowDialog() == DialogResult.OK) UpdateConfigTargetList();
    }
    /// <summary>
    /// Edit an existing target
    /// </summary>
    /// <param name="ConfigTargetID">The ConfigTargetID of the item to edit</param>
    private void EditConfigTarget(int ConfigTargetID)
    {
      ConfigTarget CT = new ConfigTarget(ConfigTargetID);
      if (CT.ShowDialog() == DialogResult.OK) UpdateConfigTargetList();
    }
    /// <summary>
    /// Delete the given ConfigTarget
    /// </summary>
    /// <param name="ConfigTargetID">The ConfigTargetID of the item to edit</param>
    private void DeleteConfigTarget(int ConfigTargetID)
    {
      if (MessageBox.Show("Are you sure you want to delete the selected cofiguration target ?", "Deletion warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == System.Windows.Forms.DialogResult.Yes)
      {
        configTargetsTA.Delete(ConfigTargetID);
        UpdateConfigTargetList();
      }
    }
    #endregion

    #region Manage Config Sets
    /// <summary>
    /// Refreshes both the ConfigSets listview and the ConfigSets combobox list from database
    /// </summary>
    private void UpdateConfigSetList()
    {
      configSetsTA.Fill(this.configDS.ConfigSets);
      lvSets.BeginUpdate();
      try
      {
        lvSets.Items.Clear();
        lvSets.ListViewItemSorter = null;
        foreach (var item in this.configDS.ConfigSets)
        {
          ListViewItem lvi = new ListViewItem();
          lvi.Text = item.ConfigSetName;
          lvi.SubItems.Add(item.IsNoteNull() ? "" : item.Note);
          lvi.Tag = item.ConfigSetID;
          lvSets.Items.Add(lvi);
        }
        cbxConfigSets.DataSource = configDS.ConfigSets.ToList();
        lvSets.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
        if (lvSets.Items.Count > 0) lvSets.Items[0].Selected = true;
      }
      finally
      {
        lvSets.EndUpdate();
      }
      UpdateSetTargets();
    }
    /// <summary>
    /// Create a new Configuration Set
    /// </summary>
    private void CreateConfigSet()
    {
      ConfigSet CS = new ConfigSet();
      if (CS.ShowDialog() == DialogResult.OK) UpdateConfigSetList();
    }
    /// <summary>
    /// Opens an existing Configuration Set to be edited
    /// </summary>
    private void EditConfigSet(int ConfigSetID)
    {
      ConfigSet CS = new ConfigSet(ConfigSetID);
      if (CS.ShowDialog() == DialogResult.OK) UpdateConfigSetList();
    }
    /// <summary>
    /// Delete an existing Configuration Set
    /// </summary>
    /// <param name="ConfigSetID"></param>
    private void DeleteConfigSet(int ConfigSetID)
    {
      if (MessageBox.Show("Are you sure you want to delete the selected cofiguration set ? All configurations, specific for this set will also be deleted !", "Data loss warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == System.Windows.Forms.DialogResult.Yes)
      {
        configSetsTA.Delete(ConfigSetID);
        UpdateConfigSetList();
      }
    }
    /// <summary>
    /// Updates the list of configuration targets included/excluded in a set 
    /// </summary>
    private void UpdateSetTargets()
    {
      lvTargetsInSet.Items.Clear();
      lvTargetsNotInSet.Items.Clear();
      if (lvSets.SelectedItems.Count > 0)
      {
        int curSetID = (int)lvSets.SelectedItems[0].Tag;
        ConfigDS.SetTargetsDataTable targets = new ConfigDS.SetTargetsDataTable();
        // list targets included current set
        setTargetsTA.FillTargetsInSet(targets, curSetID);
        lvTargetsInSet.BeginUpdate();
        try
        {
          foreach (var item in targets)
          {
            ListViewItem lvi = lvTargetsInSet.Items.Add(item.TargetName);
            lvi.Tag = item.ConfigTargetID;
          }
        }
        finally
        {
          lvTargetsInSet.EndUpdate();
        }
        if (lvTargetsInSet.Items.Count > 0) lvTargetsInSet.Items[0].Selected = true;
        // list targets not in set
        setTargetsTA.FillTargetsNotInSet(targets, curSetID);
        lvTargetsNotInSet.BeginUpdate();
        try
        {
          foreach (var item in targets)
          {
            ListViewItem lvi = lvTargetsNotInSet.Items.Add(item.TargetName);
            lvi.Tag = item.ConfigTargetID;
          }
        }
        finally
        {
          lvTargetsNotInSet.EndUpdate();
        }
        if (lvTargetsNotInSet.Items.Count > 0) lvTargetsNotInSet.Items[0].Selected = true;
      }
      RefreshTargetList();
    }
    /// <summary>
    /// Refreshes the selectable configuration target list depending on the selected configuration set
    /// </summary>
    private void RefreshTargetList()
    {
      if (cbxConfigSets.SelectedItem != null)
      {
        ConfigDS.SetTargetsDataTable setTargets = new ConfigDS.SetTargetsDataTable();
        int selSetID = ((ConfigDS.ConfigSetsRow)cbxConfigSets.SelectedItem).ConfigSetID;
        setTargetsTA.FillTargetsInSet(setTargets, selSetID);
        cbxConfigSetTargets.DataSource = setTargets.ToList();
        ShowConfiguration();
      }
    }
    private void AddTargetToSet(int ConfigTargetID, int ConfigSetID)
    {
      qTA.AddTargetToSet(ConfigSetID, ConfigTargetID);
    }
    private void RemoveTargetFromSet(int ConfigTargetID, int ConfigSetID)
    {
      if (MessageBox.Show("If you remove a target ftom a set, its associated configuration will also be deleted ? Are you sure you want to remove this target !", "Data loss warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == System.Windows.Forms.DialogResult.Yes)
      {
        qTA.RemoveTargetFromSet(ConfigSetID, ConfigTargetID);
      }
    }
    #endregion

    #region Manage Configuration Lines
    /// <summary>
    /// Displays configuration for the currently selected configuration set target
    /// </summary>
    private void ShowConfiguration()
    {
      if (targetConfigurationChanged && MessageBox.Show("Configuration has changed. Do you want to save it ?", "", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == System.Windows.Forms.DialogResult.Yes)
      {
        SaveConfigurationToDatabase(loadedConfigSetTargetID);
      }
      loadedConfigSetTargetID = -1;
      tbConfigurationLines.Text = "";
      if (cbxConfigSetTargets.SelectedItem != null)
      {
        int selConfigSetTargetID = ((ConfigDS.SetTargetsRow)cbxConfigSetTargets.SelectedItem).ConfigSetTargetID;
        configLinesTA.FillByConfigSetTargetID(this.configDS.ConfigLines, selConfigSetTargetID);
        var lines = from l in this.configDS.ConfigLines select l.ConfigLine;
        tbConfigurationLines.Lines = lines.ToArray();
        loadedConfigSetTargetID = selConfigSetTargetID;
      }
      targetConfigurationChanged = false;
    }
    /// <summary>
    /// Saves the currently displayed configuration to database
    /// </summary>
    /// <param name="ConfigurationSetTargetID">If set, configuration lines will be stored/updated for this reference, regardless the currently selected target</param>
    private void SaveConfigurationToDatabase(int ConfigurationSetTargetID = -1)
    {
      if (cbxConfigSetTargets.SelectedItem != null || ConfigurationSetTargetID != -1)
      {
        int selConfigSetTargetID = ConfigurationSetTargetID != -1 ? ConfigurationSetTargetID : ((ConfigDS.SetTargetsRow)cbxConfigSetTargets.SelectedItem).ConfigSetTargetID;
        qTA.DeleteConfiguration(selConfigSetTargetID);
        configDS.ConfigLines.Clear();
        foreach (string line in tbConfigurationLines.Lines)
        {
          ConfigDS.ConfigLinesRow clr = configDS.ConfigLines.NewConfigLinesRow();
          clr.ConfigSetTargetID = selConfigSetTargetID;
          clr.ConfigLine = line;
          configDS.ConfigLines.AddConfigLinesRow(clr);
        }
        configLinesTA.Update(configDS);
        targetConfigurationChanged = false;
      }
    }
    private void LoadConfigurationFromFile(string FileName = "")
    {
      try
      {
        openFileDialog.Filter = "TXT files|*.txt|All files|*.*";
        if (FileName != "" || openFileDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
        {
          if ((tbConfigurationLines.Text.Length == 0 & !targetConfigurationChanged) || MessageBox.Show("Overwrite configuration from file ?", "Open file", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
          {
            tbConfigurationLines.Text = "";
            tbConfigurationLines.Lines = File.ReadAllLines(FileName != "" ? FileName : openFileDialog.FileName);
          }
        }
      }
      catch (Exception Ex)
      {
        MessageBox.Show("Unfortunately an error occurred : " + Ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }
    private void SaveConfigurationToFile()
    {
      saveFileDialog.Filter = "TXT files|*.txt|All files|*.*";
      if (saveFileDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
      {
        File.WriteAllLines(saveFileDialog.FileName, tbConfigurationLines.Lines);
      }
    }
    #endregion

    #region Event handlers
    private void DrawGradientBackground(object sender, PaintEventArgs e)
    {
      Rectangle rect = (sender as Control).ClientRectangle;
      using (Brush br = new LinearGradientBrush(rect, System.Drawing.Color.White, (sender as Control).BackColor, LinearGradientMode.Vertical))
      {
        e.Graphics.FillRectangle(br, rect);
      }
    }
    private void btnAddTarget_Click(object sender, EventArgs e)
    {
      AddConfigTarget();
    }
    private void btnDeleteTarget_Click(object sender, EventArgs e)
    {
      if (lvTargets.SelectedItems.Count > 0)
      {
        ListViewItem lvi = lvTargets.SelectedItems[0];
        if (lvi != null) DeleteConfigTarget((int)lvi.Tag);
      }
    }
    private void btnEditTarget_Click(object sender, EventArgs e)
    {
      if (lvTargets.SelectedItems.Count > 0)
      {
        ListViewItem lvi = lvTargets.SelectedItems[0];
        if (lvi != null) EditConfigTarget((int)lvi.Tag);
      }
    }
    private void lvTargets_DoubleClick(object sender, EventArgs e)
    {
      if (lvTargets.SelectedItems.Count > 0)
      {
        ListViewItem lvi = lvTargets.SelectedItems[0];
        if (lvi != null) EditConfigTarget((int)lvi.Tag);
      }
    }
    private void lvSets_MouseDoubleClick(object sender, MouseEventArgs e)
    {
      if (lvSets.SelectedItems.Count > 0)
      {
        ListViewItem lvi = lvSets.SelectedItems[0];
        if (lvi != null) EditConfigSet((int)lvi.Tag);
      }
    }
    private void btnCreateSet_Click(object sender, EventArgs e)
    {
      CreateConfigSet();
    }
    private void btnDeleteSet_Click(object sender, EventArgs e)
    {
      if (lvSets.SelectedItems.Count > 0)
      {
        ListViewItem lvi = lvSets.SelectedItems[0];
        if (lvi != null) DeleteConfigSet((int)lvi.Tag);
      }
    }
    private void btnEditSet_Click(object sender, EventArgs e)
    {
      if (lvSets.SelectedItems.Count > 0)
      {
        ListViewItem lvi = lvSets.SelectedItems[0];
        if (lvi != null) EditConfigSet((int)lvi.Tag);
      }
    }
    private void lvSets_SelectedIndexChanged(object sender, EventArgs e)
    {
      UpdateSetTargets();
    }
    private void btnAddSetTarget_Click(object sender, EventArgs e)
    {
      if (lvSets.SelectedItems.Count > 0 & lvTargetsNotInSet.SelectedItems.Count > 0)
      {
        ListViewItem targetLVI = lvTargetsNotInSet.SelectedItems[0];
        ListViewItem setLVI = lvSets.SelectedItems[0];
        AddTargetToSet((int)targetLVI.Tag, (int)setLVI.Tag);
        UpdateSetTargets();
      }
    }
    private void btnRemoveSetTarget_Click(object sender, EventArgs e)
    {
      if (lvSets.SelectedItems.Count > 0 & lvTargetsInSet.SelectedItems.Count > 0)
      {
        ListViewItem targetLVI = lvTargetsInSet.SelectedItems[0];
        ListViewItem setLVI = lvSets.SelectedItems[0];
        RemoveTargetFromSet((int)targetLVI.Tag, (int)setLVI.Tag);
        UpdateSetTargets();
      }
    }
    private void cbxConfigSets_Format(object sender, ListControlConvertEventArgs e)
    {
      e.Value = ((ConfigDS.ConfigSetsRow)e.ListItem).ConfigSetName;
    }
    private void cbxConfigSetTargets_Format(object sender, ListControlConvertEventArgs e)
    {
      e.Value = ((ConfigDS.SetTargetsRow)e.ListItem).TargetName;
    }
    private void cbxConfigSets_DropDownClosed(object sender, EventArgs e)
    {
      RefreshTargetList();
    }
    private void cbxConfigSetTargets_DropDownClosed(object sender, EventArgs e)
    {
      ShowConfiguration();
    }
    private void btnSaveConfiguration_Click(object sender, EventArgs e)
    {
      SaveConfigurationToDatabase();
    }
    private void tbConfigurationLines_TextChanged(object sender, EventArgs e)
    {
      targetConfigurationChanged = true;
    }
    private void btnDeploySet_Click(object sender, EventArgs e)
    {
      if (lvSets.SelectedItems.Count > 0)
      {
        ListViewItem setLVI = lvSets.SelectedItems[0];
        Deployer d = new Deployer((int)setLVI.Tag);
        if (d.ShowDialog() == System.Windows.Forms.DialogResult.OK) this.MdiParent.LayoutMdi(MdiLayout.Cascade);
      }
    }
    private void btnPullConfig_Click(object sender, EventArgs e)
    {
      if (lvSets.SelectedItems.Count > 0)
      {
        ListViewItem setLVI = lvSets.SelectedItems[0];
        ConfigPuller cp = new ConfigPuller((int)setLVI.Tag);
        cp.Show();
      }
    }
    private void tbConfigurationLines_DragDrop(object sender, DragEventArgs e)
    {
      string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
      if (files.Length > 0)
      {
        LoadConfigurationFromFile(files[0]);
      }
    }
    private void tbConfigurationLines_DragEnter(object sender, DragEventArgs e)
    {
      if (e.Data.GetDataPresent(DataFormats.FileDrop)) e.Effect = DragDropEffects.Copy;
    }
    private void tbConfigurationLines_DragOver(object sender, DragEventArgs e)
    {
      if (e.Data.GetDataPresent(DataFormats.FileDrop)) e.Effect = DragDropEffects.Copy;
    }
    private void btnLoadFromFile_Click(object sender, EventArgs e)
    {
      LoadConfigurationFromFile();
    }
    private void btnSaveToFile_Click(object sender, EventArgs e)
    {
      SaveConfigurationToFile();
    }
    #endregion



  }
}
