﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Windows.Forms;
using System.ComponentModel;

namespace PGT.ConfigurationManager
{
  /// <summary>
  /// This static class is used for creating new database and metadata for new installations
  /// </summary>
  static class DatabaseManager
  {
    /// <summary>
    /// Checks database and creates it if missing and also creates required DB objects
    /// </summary>
    /// <param name="ServerName">The SQL Server name to connect to</param>
    /// <param name="DatabaseName">The name of the database to check or create</param>
    /// <param name="intSecurity">If integrated security to be used for connection</param>
    /// <param name="sqlUserName">SQL Username for connection</param>
    /// <param name="sqlPassword">SQL Password for connection</param>
    /// <param name="animation">A Backgoundworker to stop before displaying any dialog boxes. Optional</param>
    /// <returns></returns>
    public static bool RegisterDatabase(string ServerName, string DatabaseName, bool intSecurity, string sqlUserName, string sqlPassword, BackgroundWorker animation = null)
    {
      bool runResult = true;
      bool DBExists = false;
      bool DBCreated = false;
      bool DBObjectsCreated = false;
      try
      {
        #region Create Database if not exists
        string MasterConnectionString = intSecurity ? string.Format("Data Source={0};Initial Catalog=master;Integrated Security=True", ServerName) :
          string.Format("Data Source={0};Initial Catalog=master;Persist Security Info=True;User ID={1};Password={2}", ServerName, sqlUserName, sqlPassword);
        Debug.WriteLine(string.Format("Connecting to master db using : {0}", MasterConnectionString));
        using (SqlConnection conn = new SqlConnection(MasterConnectionString))
        {
          try
          {
            conn.Open();
            #region Test database existence
            try
            {
              Debug.WriteLine(string.Format("Checking database : {0}", DatabaseName));
              SqlCommand scmd = conn.CreateCommand();
              scmd.CommandText = string.Format("SELECT name FROM master.sys.databases WHERE name = N'{0}'", DatabaseName);
              object o = new object();
              o = scmd.ExecuteScalar();
              DBExists = o != null;
              if (!DBExists)
              {
                Debug.WriteLine("database doesn't exist");
                if (animation != null) animation.CancelAsync();
                runResult = MessageBox.Show(string.Format("Database {0} does not exists. Do you want to create it now?", DatabaseName), "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes;
                if (runResult & animation != null && !animation.IsBusy) animation.RunWorkerAsync();
              }
              else Debug.WriteLine("database exists");
            }
            catch (SqlException Ex)
            {
              Debug.WriteLine("Unexpected error 1");
              if (animation != null) animation.CancelAsync();
              MessageBox.Show(string.Format("Unexpected error while checking database {0} : {1} ", DatabaseName, Ex.Message), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
              runResult = false;
            }
            #endregion
            if (runResult & !DBExists)
            {
              #region Create database
              try
              {
                Debug.WriteLine(string.Format("Creating database : {0}", DatabaseName));
                SqlCommand scmd = conn.CreateCommand();
                scmd.CommandText = "CREATE DATABASE " + DatabaseName;
                scmd.ExecuteNonQuery();
                scmd.CommandText = string.Format("ALTER DATABASE {0} SET AUTO_SHRINK ON", DatabaseName);
                scmd.ExecuteNonQuery();
                DBCreated = true;
                Debug.WriteLine("database created successfully");
              }
              catch (SqlException Ex)
              {
                if (((SqlException)Ex).Number == 1801)
                {
                  Debug.WriteLine("database already exists");
                }
                else
                {
                  Debug.WriteLine("Unexpected error 1");
                  if (animation != null) animation.CancelAsync();
                  MessageBox.Show(string.Format("Unexpected error while creating database {0} : {1} ", DatabaseName, Ex.Message), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                  runResult = false;
                }
              }
              #endregion
            }
          }
          catch (SqlException Ex)
          {
            Debug.WriteLine("Unexpected error 1");
            if (animation != null) animation.CancelAsync();
            MessageBox.Show(string.Format("Unexpected error while opening connection : {0} ", Ex.Message), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            runResult = false;
          }
        }
        #endregion

        if (runResult)
        {
          #region Connect to database and create database objects if missing

          string InstallerConnectionString = intSecurity ? string.Format("Data Source={0};Initial Catalog={1};Integrated Security=True", ServerName, DatabaseName) :
            string.Format("Data Source={0};Initial Catalog={1};Persist Security Info=True;User ID={1};Password={2}", ServerName, DatabaseName, sqlUserName, sqlPassword);
          Debug.WriteLine(string.Format("Connecting using : {0}", InstallerConnectionString));
          using (SqlConnection conn = new SqlConnection(InstallerConnectionString))
          {
            conn.Open();
            SqlCommand scmd;
            try
            {
              try
              {
                #region Check if require objects exist
                Debug.WriteLine("Checking database objects");
                bool RequiredObjectsExist = false;
                // This is to try whether required database objects exist or not
                scmd = new SqlCommand("SELECT COUNT (*) FROM CONFIGTARGETS", conn);
                try
                {
                  object o = new object();
                  o = scmd.ExecuteScalar();
                  RequiredObjectsExist = true;
                }
                catch (SqlException Ex)
                {
                  // Error number 208 is normal, meaning that there is no Params table yet
                  if (((SqlException)Ex).Number == 208)
                  {
                    Debug.WriteLine("Error 208 : there is no CONFIGTARGETS table yet");
                  }
                  else
                  {
                    // other exceptions are unexpected and unhandled errors
                    if (animation != null) animation.CancelAsync();
                    MessageBox.Show(string.Format("Unexpected error while creating database {0} : {1} ", DatabaseName, Ex.Message), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    runResult = false;
                    throw Ex;
                  }
                }
                #endregion

                #region Run DDL script if required objects do not exist
                if (!RequiredObjectsExist)
                {
                  // this is a new empty database
                  Debug.WriteLine("this is a new empty database, running DDL script");
                  try
                  {
                    string SqlScript = Resource1.CreateConfigManagerObjects_20151007_v1;
                    SqlScript = SqlScript.Replace("GO", "^");
                    string[] SqlCommands = SqlScript.Split("^".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);
                    foreach (string command in SqlCommands)
                    {
                      Debug.WriteLine(scmd.CommandText);
                      scmd = new SqlCommand(command, conn);
                      scmd.ExecuteNonQuery();
                    }
                    DBObjectsCreated = true;
                    Debug.WriteLine("DB objects created sucessfully");
                  }
                  catch (Exception Ex)
                  {
                    if (animation != null) animation.CancelAsync();
                    MessageBox.Show(string.Format("Unexpected error while creating database objects : {0} ", Ex.Message), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    runResult = false;
                  }
                }
                #endregion
              }
              finally
              {
                conn.Close();
              }
            }
            catch (Exception ex)
            {
              MessageBox.Show(ex.Message, "Sorry, I am unable to register database. Please view debug logs for more details.", MessageBoxButtons.OK, MessageBoxIcon.Error);
              runResult = false;
            }
          }
          #endregion
        }
      }
      catch (Exception ex)
      {
        if (animation != null) animation.CancelAsync();
        MessageBox.Show("Sorry, I am unable to register database. Please view debug logs for more details. Error : " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        runResult = false;
      }
      if (DBCreated | DBObjectsCreated)
      {
        if (animation != null) animation.CancelAsync();
        string s = DBCreated ? "Database created sucessfully." : "";
        if (DBObjectsCreated) s += " Required database objects created sucessfully.";
        MessageBox.Show(s, "SQL Operation success", MessageBoxButtons.OK, MessageBoxIcon.Information);
      }
      return runResult;
    }
  }
}
