#  ----------------------------------------------------------------
# Copyright 2016 Laszlo Frank
#
# This is a modified code of the original NetConfProvider class to
# provide the same functionality using PGT connection services and 
# CLR objects.
#
# ------------------------------------------------------------------
#  ------------------------ Original Code ---------------------------
# Copyright 2016 Cisco Systems
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ------------------------------------------------------------------
import clr
clr.AddReferenceToFileAndPath("C:\\My Projects\\Visual Studio Projects\\PrettyGoodTerminal\\bin\\STerminal.dll")
import PGT
from PGTXmlEncoder import XmlEncoder
from PGTXmlDecoder import XmlDecoder
from xml.etree import ElementTree
from ydk._core._dm_meta_info import REFERENCE_IDENTITY_CLASS, REFERENCE_ENUM_CLASS
from ydk.errors import YPYServiceProviderError, YPYErrorCode
from ydk.types import Empty, DELETE, READ, Decimal64, YList, YListItem, YLeafList
class PGTNetconfServiceProvider(object):
  """ 
Pretty Good Terminal .NET based Netconf ServiceProvider.
A wrapper component around PGT's NetconfTerminal object injected into context when connected over Netconf protocol.
IronPython and PGT cannot make use of original Cisco code due to .NET / native code interoperability issues.
PGTNetconfServiceProvider class therefore mimics the functionality of NetConfServiceProvider class in original Cisco YDK API
and provides the same services.
  """
  def __init__(self, _terminal):
    if _terminal == None :
      self.ncTerminal = PGT.NetconfTerminal(60, 830)
    elif _terminal.GetType().Name == "NetconfTerminal":
      self.ncTerminal = _terminal
    else :
      TypeError("Invalid argument")
      
  def _attach_tag(self, root, entity, optype):
    if type(entity) ==  YList or type(entity) == YLeafList:
      self._attach_list_tag(root, entity, optype)
      
    elif type(entity) == YListItem:
      # attach tag to this particular leaflist element
      chs = root.getchildren()
      for ch in chs:
        # match parent
        if entity.parent.i_meta.yang_name == ch.tag:
          ch.clear()
          elem = ElementTree.SubElement(ch, entity.name)
          xc = 'urn:ietf:params:xml:ns:netconf:base:1.0'
          elem.set('{' + xc + '}operation', 'delete')
          elem.text = str(entity.item)
    else:
      chs = root.getchildren()
      for ch in chs:
        if entity.i_meta.yang_name == ch.tag:
          elem = ch
          xc = 'urn:ietf:params:xml:ns:netconf:base:1.0'
          elem.set('{' + xc + '}operation', 'delete')   
          
  def _attach_list_tag(self, root, entity, optype):
    if type(entity) == YList or type(entity) == YLeafList:
      for item in entity:
        self._attach_list_tag(root, item, optype)
    else:
      chs = root.getchildren()
      for ch in chs:
        elem = self._match_key(ch, entity)
        if elem is not None:
          xc = 'urn:ietf:params:xml:ns:netconf:base:1.0'
          elem.set('{' + xc + '}operation', 'delete')             
    
  def _check_read_only_edit_error(self, entity):
    if type(entity) ==  YLeafList:
      pass
    elif type(entity) == YListItem:
      pass
    elif isinstance(entity, YList):
      for item in entity:
        _check_read_only_edit_error(item)
    else:
      if not entity.is_config():
        _raise_read_only_edit_error()      
  def _connect(self, Host, UserName, Password):
    if self.ncTerminal.IsConnected() :
      # Not doing anything as already connected
      pass
    else:
      self.ncTerminal.Connect("SSH", Host, UserName, Password)
  def close(self):
    self.ncTerminal.Close()
    
  def _create_element(self, root, oper, target, candidate, config_or_filter, optype):
    root = ElementTree.SubElement(root, oper)
    if target is not None:
      target = ElementTree.SubElement(root, target)
    if candidate is not None:
      if target is not None:
        candidate = ElementTree.SubElement(target, candidate)
      else:
        candidate = ElementTree.SubElement(root, candidate)
        
    if operation_is_edit(optype):
      NSMAP = {"xc": "urn:ietf:params:xml:ns:netconf:base:1.0"}
      root = ElementTree.SubElement(root, 'config', attrib=NSMAP)
    elif optype == 'EVENT_SUBSCRIBE':
      root = ElementTree.SubElement(candidate, config_or_filter)
      root.set('xmlns', "urn:ietf:params:xml:ns:netconf:base:1.0")
    elif optype == 'READ':
      root = ElementTree.SubElement(root, config_or_filter)
      root.set('type', "subtree")
    else:
      NSMAP = {"xc", "urn:ietf:params:xml:ns:netconf:base:1.0"}
      root = ElementTree.SubElement(root, config_or_filter, attrib=NSMAP)
    return root
    
  def _create_top_level_entity_from_read_filter(self, read_filter):
    non_list_filter = read_filter
    while isinstance(non_list_filter, YList):
      non_list_filter = non_list_filter.parent
    if non_list_filter is None:
      #self.crud_logger.error('Cannot determine hierarchy for entity. Please set the parent reference')
      raise YPYServiceProviderError('Cannot determine hierarchy for entity. Please set the parent reference')
    top_entity_meta_info = non_list_filter._meta_info()
    while hasattr(top_entity_meta_info, 'parent') and top_entity_meta_info.parent is not None:
      # need to find the member that has
      top_entity_meta_info = top_entity_meta_info.parent
    exec_import = 'from ' + top_entity_meta_info.pmodule_name + ' import ' + top_entity_meta_info.name.split('.')[0]
    exec exec_import
    entity = eval(top_entity_meta_info.name + '()')
    return entity 
     
  def _create_preamble(self, entity, root):
    if type(entity) == YLeafList or type(entity) == YListItem:
      # escape current level
      entity = entity.parent     
      
    elif type(entity) == YList:
      entity = entity[0]
      
    parent_meta_tuple_list = self._get_parent_tuple_list(entity, entity._meta_info())
    parent_ns = self._get_parent_namespace(root) 
    return self._encode_parents_of_root(root, parent_meta_tuple_list, parent_ns)         
    
  def _create_root(self):
    NSMAP = {'xmlns': 'urn:ietf:params:xml:ns:netconf:base:1.0'}   
    self.head = ElementTree.Element('rpc', NSMAP)
    
    self.head.set('message-id', '101')
    return self.head       
  def encode(self, entity, operation, only_config=False):
    #return self.ncTerminal.Encode(entity, operation, only_config)
    root = self._create_root()
    if operation_is_edit(operation):
      root = self._encode_edit_request(root, entity, operation)
      
    else:
      root = self._encode_read_request(root, entity, operation, only_config)
      
    #payload = ElementTree.tostring(self.head, method='xml', pretty_print='True')
    payload = ElementTree.tostring(self.head, encoding='utf-8', method='xml')
    return payload
    
  def _encode_edit_request(self, root, entity, optype):
    self._check_read_only_edit_error(entity)
    root = self._create_element(root,'edit-config','target', self._get_target_datastore(),'config', optype)
    root = self._create_preamble(entity, root)
    self._encode_epilogue(entity, root, optype)
    if not operation_is_create_or_update(optype):
      self._attach_tag(root, entity, optype)
      return root
    
  def _encode_epilogue(self, entity, root, optype):
    if type(entity) == YLeafList or type(entity) == YListItem:
      # parent_meta_tuple_list is not created for leaflist's parent
      entity = entity.parent
      XmlEncoder().encode_to_xml(entity, root, optype)
      
    elif type(entity) == YList:
      for item in entity:
          self._encode_epilogue(item, root, optype)
    else:
      XmlEncoder().encode_to_xml(entity, root, optype)    
  def _encode_keys(self, root, entity, meta_info):
    for key in meta_info.key_members():
      self._encode_key(root, entity, meta_info, key)
    return root
    
  def _encode_key(self, root, entity, meta_info, key):
    key_value = getattr(entity, key.presentation_name)
    if key_value is None:
      self._raise_key_missing_error()
      return
    if key.mtype == REFERENCE_IDENTITY_CLASS:
      # encode an identity
      NSMAP = {'idx' : _yang_ns._namespaces[key_value._meta_info().module_name]}
      member_elem = ElementTree.SubElement(root, key.name, attrib=NSMAP)
      member_elem.text = 'idx:%s' % key_value._meta_info().yang_name
    else:
      member_elem = ElementTree.SubElement(root, key.name)
      if key.mtype == REFERENCE_ENUM_CLASS:
          key_value = key_value.name.replace('_', '-').lower()
      member_elem.text = str(key_value)
    
  def _encode_parents_of_root(self, root, parent_meta_tuple_list, parent_ns):
    for meta_info, parent in reversed(parent_meta_tuple_list):
      root = ElementTree.SubElement(root, meta_info.yang_name)
      ns = meta_info.namespace
      if ns is not None and parent_ns != ns:
        root.set('xmlns', ns)
        parent_ns = ns
      root = _encode_keys(root, parent, meta_info)
    return root    
  def _encode_read_request(self, root, entity, optype, only_config):
    get_str = 'get'
    source = None
    source_ds = None
    if only_config:
      get_str = 'get-config'
      source = 'source'
      source_ds = 'running'
    root = self._create_element(root, get_str, source, source_ds, 'filter', optype)
    root.set('type', "subtree")
    root = self._create_preamble(entity, root)
    XmlEncoder().encode_filter(entity, root, optype)
    return root
  def _entity_is_abstract(self, entity, meta_info):
    return meta_info.is_abstract and entity.parent is None
  def _entity_has_no_keys(self, entity, meta_info):
    return entity is None and len(meta_info.key_members()) > 0
    
  def _get_parent_tuple_list(self, current_parent, current_meta_info):
    parent_meta_tuple_list = []
    while hasattr(current_meta_info, 'parent'):
      current_meta_info = current_meta_info.parent
      if current_parent is not None:
        current_parent = current_parent.parent
      if _entity_is_abstract(current_parent, current_meta_info) or _entity_has_no_keys(current_parent, current_meta_info):
        self._raise_parent_hierarchy_error()
        return []
      parent_meta_tuple_list.append((current_meta_info, current_parent))
    return parent_meta_tuple_list
    
  def _get_parent_namespace(self, current_parent):
    parent_ns = None
    while current_parent is not None and parent_ns is None:
      parent_ns = current_parent.get('xmlns')
      #current_parent = current_parent.getparent()
      # this does not work : current_parent = current_parent.find("..")
      current_parent = self._get_element_parent(current_parent)
    return parent_ns   
    
  def _get_element_parent(self, element): 
    for thisElement in self.head.getiterator():
      for thisChild in thisElement._children:
        if thisChild == element : return thisElement
        
    return None       
    
  def _get_target_datastore(self):
    assert self.ncTerminal is not None
    target_ds = 'candidate'
    if not confirmed_commit_supported(self.ncTerminal):
      target_ds = 'running'
    return target_ds
  def decode(self, payload, read_filter):
    if read_filter is None:
        return XmlDecoder().decode(payload)
        
    # In order to figure out which fields are the
    # ones we are interested find the field list
    entity = self._create_top_level_entity_from_read_filter(read_filter)
    # XmlDecoder._bind_to_object(payload, entity, self._nc_manager.server_capabilities)
    XmlDecoder._bind_to_object(payload, entity, self.ncTerminal.GetServerCapabilities())
    
    # drill down to figure out the field access expression
    # that matches the entity or entities to be returned
    # not the argument passed in as a filter might have
    # incomplete key paths, in which case what is returned
    # will be the entity whose common path can be determined
    
    current = entity
    current_entity = read_filter
    current_meta = current_entity.i_meta
    yang_nodes = []
    while hasattr(current_meta, 'parent'):
        yang_nodes.append(current_meta.yang_name)
        current_meta = current_meta.parent
    if current_meta:
        yang_nodes.append(current_meta.yang_name)
    yang_nodes = list(reversed(yang_nodes))
    yang_nodes = yang_nodes[1:]
    for yang_node_name in yang_nodes:
      found = False
      for member in current._meta_info().meta_info_class_members:
        if member.name == yang_node_name:
          found = True
          current = eval('current.%s' % member.presentation_name)
          if current is None:
            return None
          if isinstance(current, YList):
            if len(current) == 0:
                return None
            if len(current) > 2:
                return current
            if len(current) == 1:
                current = current[0]
          break
      if not found:
          #self.crud_logger.error('Error determing what needs to be returned')
          raise YPYServiceProviderError('Error determining what needs to be returned')
    return current
  def execute(self, payload, operation):
    reply = self.ncTerminal.ExecCommand(payload, operation)
    return reply
  def _get_capabilities(self):
    capabilities = self.ncTerminal.GetServerCapabilities()
    return capabilities
    
  def _match_key(self, root, entity):
    if type(entity) == YListItem:
      return self._match_leaflist_key(root, entity)
    else:
      return self._match_list_key(root, entity)   
  def _match_leaflist_key(self, root, entity):
    target_root = None
    chs = root.getchildren()
    # leaflist of enum
    if hasattr(entity, 'i_meta') and entity.i_meta.mtype == REFERENCE_ENUM_CLASS:
      value = key_value.name.replace('_', '-').lower()
      
    value = str(entity.item)
    for ch in chs:
      if ch.tag == entity.name and ch.text == value:
        target_root = ch
        return target_root
        
    return target_root
  def _match_list_key(self, root, entity):
    target_root = None
    chs = root.getchildren()
    keys = entity.i_meta.key_members()
    for key in keys:
      key_value = getattr(entity, key.presentation_name)
      if key.mtype == REFERENCE_ENUM_CLASS:
        key_value = key_value.name.replace('_', '-').lower()
      key_value = str(key_value)
      for ch in chs:
        if key.name == ch.tag and key_value == ch.text:
          target_root = root
          return target_root
    return target_root
           
def confirmed_commit_supported(session_manager):
  capabilities = session_manager.GetServerCapabilities()
  confirmed_1_0 = 'urn:ietf:params:netconf:capability:confirmed-commit:1.0'
  confirmed_1_1 = 'urn:ietf:params:netconf:capability:confirmed-commit:1.1'
  return confirmed_1_0 in capabilities or confirmed_1_1 in capabilities
def operation_is_edit(operation):
  return operation in ('CREATE', 'UPDATE', 'DELETE')  
  
def operation_is_create_or_update(operation):
  return operation in ('CREATE', 'UPDATE')   
def _raise_parent_hierarchy_error(self):
  self.netconf_sp_logger.error(YPYErrorCode.INVALID_HIERARCHY_PARENT)
  raise YPYServiceProviderError(YPYErrorCode.INVALID_HIERARCHY_PARENT)
def _raise_key_missing_error(self):
  self.netconf_sp_logger.error(YPYErrorCode.INVALID_HIERARCHY_KEY)
  raise YPYServiceProviderError(YPYErrorCode.INVALID_HIERARCHY_KEY)
def _raise_read_only_edit_error(self):
  self.netconf_sp_logger.error(YPYErrorCode.INVALID_MODIFY)
  raise YPYServiceProviderError(YPYErrorCode.INVALID_MODIFY)
def _raise_non_rpc_error(self):
  self.netconf_sp_logger.error(YPYErrorCode.INVALID_RPC)
  raise YPYServiceProviderError(YPYErrorCode.INVALID_RPC)